/*************************************************** */
/* Rule Set Based Access Control                     */
/*                                                   */
/* Author and (c) 1999-2006: Amon Ott <ao@rsbac.org> */
/*                                                   */
/* Last modified: 22/Feb/2006                        */
/*************************************************** */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <rsbac/types.h>
#include <rsbac/syscalls.h>
#include <rsbac/error.h>
#include <rsbac/helpers.h>
#include "nls.h"
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

char *progname;

void use(void)
{
printf(gettext("RSBAC %s\n"
			"USAGE: %s [OPTION]... user\n"
			"Remove all groups and memberships from a user\n"),
			VERSION, progname);
	printf(gettext("\nOPTIONS:\n"
			"  -y\t\t\talways answer yes without asking\n"
			"  -N ta\t\t\tspecify ta transaction number\n"
			"\t\t\t(defaults to RSBAC_TA if set, else 0)\n"));
}

int main(int argc, char **argv)
{
	int res = 0;
	rsbac_list_ta_number_t ta_number = 0;
	int allyes = 0;
	rsbac_um_set_id_t um_set = RSBAC_UM_SET_AUTO;

	locale_init();
	progname = argv[0];

	{
		char *env = getenv("RSBAC_TA");

		if (env)
			ta_number = strtoul(env, 0, 0);
	}
	while ((argc > 1) && (argv[1][0] == '-')) {
		char *pos = argv[1];
		pos++;
		while (*pos) {
			switch (*pos) {
			case 'h':
				use();
				return 0;
			case 'y':
				allyes = 1;
				break;

			case 'U':
				if (argc > 2) {
					um_set =
					    strtoul(argv[2], 0, 10);
					argc--;
					argv++;
				} else {
					fprintf(stderr,
						gettext
						("%s: missing UM virtual set number value for parameter %c\n"),
						progname, *pos);
					exit(1);
				}
				break;
			case 'N':
				if (argc > 2) {
					ta_number =
					    strtoul(argv[2], 0, 10);
					argc--;
					argv++;
				} else {
					fprintf(stderr,
						gettext
						("%s: missing transaction number value for parameter %c\n"),
						progname, *pos);
					exit(1);
				}
				break;

			default:
				fprintf(stderr,
					gettext
					("%s: unknown parameter %c\n"),
					progname, *pos);
				exit(1);
			}
			pos++;
		}
		argv++;
		argc--;
	}
	if (argc > 1) {
		struct passwd *user_info_p;
		struct rsbac_acl_syscall_arg_t arg;
		char yn;

		if (rsbac_get_uid_name(ta_number,
					&arg.tid.user.uid,
					NULL,
					argv[1],
					um_set)) {
			fprintf(stderr, gettext("%s: Invalid User %s!\n"),
				progname, argv[1]);
			exit(1);
		}
		arg.tid.user.set = um_set;
		arg.target = T_USER;
		arg.subj_type = ACLS_USER;
		arg.subj_id.user = arg.tid.user;
		if (!allyes) {
			printf(gettext
			       ("Remove all groups and memberships of user %u '%s' [y/n]\n"),
			       arg.tid.user.uid, argv[1]);
			yn = getchar();
			if (yn != 'y')
				exit(0);
		}
		res = rsbac_acl(ta_number, ACLC_remove_user, &arg);
		error_exit(res);
		exit(0);
	} else {
		use();
		exit(1);
	}
}
